<?php 
/*
** Store Owner Shipment Options Template
*/

/* 
**========== Direct access not allowed =========== 
*/
if( ! defined('ABSPATH' ) ){ exit; }

$service_generator = array();
$wc_order = wc_get_order($order_id);
$shipping_service = get_post_meta( $order_id, 'postis_shipping_method', true );

$shipping_country = $wc_order->get_shipping_country();
$shipping_state = $wc_order->get_shipping_state();
$shipping_postal_code = $wc_order->get_shipping_postcode();

// getting Packages Options
$all_packages = postis_package_deliver_fail_options();

// Getting Content Delivery Options
$content_delivery = postis_content_delivery_options();

// Check if shipmentis international
$is_shipment_international = postis_is_shipment_international($shipping_country);

$get_all_services = isset($shipping_options['deliveryServicesAndPrices']) && !empty($shipping_options['deliveryServicesAndPrices']) ? $shipping_options['deliveryServicesAndPrices'] : array();

foreach ($get_all_services as $index => $service_meta) {

    if ( isset($service_meta['deliveryServiceId']) && $service_meta['deliveryServiceId'] != '' && $service_meta['deliveryServiceId'] == $shipping_service ) {
        
        $optional_services = isset($service_meta['optionalServices']) ? $service_meta['optionalServices'] : array();
        
        foreach ($optional_services as $option_name => $val) {
            
            if ($val) {
                
                switch ($option_name) {
                    case 'insurance':
                       $service_generator['insurance']       = array('type'=>'checkbox', 'attach_with'=>'insuranceAmount','show'=> true,'required'=> false);
                       $service_generator['insuranceAmount'] = array('type'=>'text', 'attach_with'=>'no', 'show'=> false, 'required'=> true);
                        break;
                    case 'cod':
                    case 'reference':
                       $service_generator['cod']       = array('type'=>'checkbox', 'attach_with'=>'reference','show'=> true, 'required'=> false);
                       $service_generator['reference'] = array('type'=>'text', 'attach_with'=>'no','show'=> false,'required'=> false);
                        break;
                    
                    default:
                        $service_generator[$option_name] = array('type'=>'checkbox', 'attach_with'=>'no' ,'show'=> true, 'required'=> false);
                        break;
                }
            }
        }
    }
}

?>

<div class="postis-admin-shipment-options-wrapper">
    
    <form class="postis-submit-shipment-options">
        
        <input type="hidden" name="action" value="postis_create_shipment_action"/>
        <input type="hidden" name="order_id" value="<?php echo esc_attr($order_id); ?>"/>
        
        <div class="postis-international-options-section">
            <h1><?php _e('Optional Services', "postis"); ?></h1>
            
            <span class="postis-cod-alert-msg" style="display:none;"><?php _e('To use this service you must provide bank details to Posturinn', "postis"); ?></span>
            
            <table class="table">
                <tbody>
                    <?php 
                    foreach ($service_generator as $option_name => $option_meta) {
                    ?>
                        <tr class="option-line-<?php echo esc_attr($option_name); ?>" data-attach_with="<?php echo esc_attr($option_meta['attach_with']); ?>" style="display:<?php if(!$option_meta['show']) echo 'none;'; ?>">
                            <th class="opt-header">
                                <label><?php echo __($option_name,"postis"); ?></label>
                            </th>
                            <td>
                                <input 
                                    type="<?php echo esc_attr($option_meta['type']); ?>" 
                                    class="postis-attach-field-js postis-shipment-<?php echo esc_attr($option_name); ?>" 
                                    name="optional_services[<?php echo esc_attr($option_name);?>]" 
                                    <?php if(!$option_meta['show']) echo 'disabled' ?> 
                                    <?php if($option_meta['required']) echo 'required="true"'; ?> 
                                >
                            </td>
                        </tr>            
                    <?php
                    }
                    ?>
                    
                    <tr>
                        <th class="opt-header">
                            <label><?php _e('Number Of Parcels', "postis"); ?></label> 
                        </th>
                        <td>
                            <input type="number" min="1" id="postis-numberofitems-id" name="numberofitems" value="1">
                        </td>
                    </tr>
                    
                    <tr>
                        <th class="opt-header">
                            <label><?php _e('Description', "postis"); ?></label> 
                        </th>
                        <td>
                            <input type="text" id="postis-desc-id" name="description">
                        </td>
                    </tr>
                    
                    <!--Invalid PhoneNumber Input-->
                    <tr class="postis-phonenumber-section" style="display:none;">
                        <th></th>
                        <td>
                             <span style="border-left: 4px solid #ffba00; padding: 3px 7px; margin: 1em 0;display: block;">
                                <?php _e('The phone number in the order is not correct or missing. You can write new phone number in the field below and click "Create shipment" button.', "postis"); ?>
                            </span>
                        </td>
                    </tr>
                    <tr class="postis-phonenumber-section" style="display:none;">
                        <th class="opt-header">
                           
                            <label><?php _e('Enter Valid PhoneNumber', "postis"); ?></label> 
                        </th>
                        <td>
                            <input type="tel" id="postis-phonenumber-valid" name="phonenumber" style="border: 1px solid rgb(241, 5, 5);" placeholder="New phonenumber" disabled/>
                        </td>
                    </tr>
                </tbody>            
            </table>
            
            <?php
            if ($is_shipment_international) {
                $contents = postis_international_contents_fields();
            ?>
            <div class="postis-inter-contents-options">
                
                <h1><?php _e('Contents', "postis"); ?></h1>
                <?php 
                foreach ( $wc_order->get_items() as $item_id => $item ) {
                    $product_name = $item->get_name();
                    $product_id = $item->get_product_id();
                    $product_price_vat = wc_get_price_including_tax( $item->get_product() );
                    $quantity = $item->get_quantity();
                    $currency = get_woocommerce_currency();

                    $item_tnumber      = get_post_meta($product_id, 'hsTariffNumber', true );
                    $item_desc_content = get_post_meta($product_id, 'descriptionOfContents', true );
                ?>
                <div class="tariff-container">
                    <h3><?php echo $item->get_name(); ?></h3>
                    <table class="table postis-international-contents">
                        <thead>
                            <tr>
                                <th><?php _e('Description', "postis"); ?></th>
                                <th><?php _e('QTY', "postis"); ?></th>
                                <th><?php _e('Value', "postis"); ?></th>
                                <th><?php _e('Currency', "postis"); ?></th>
                                <th><?php _e('TariffNumber', "postis"); ?></th>
                                <th><?php _e('Country', "postis"); ?></th>
                                <th></th>
                            </tr>
                        </thead>
                        
                        <tr>
                        <?php
                            foreach ($contents as $contents_id => $contents_val) {
                                
                                $label   = isset($contents_val['label']) ? $contents_val['label'] : '';
                                $default = isset($contents_val['default']) ? $contents_val['default'] : '';
                                $req     = (isset($contents_val['required']) && $contents_val['required'] =='yes' ) ? 'required' : '' ;
                                $req     = (isset($contents_val['required']) && $contents_val['required'] =='yes' ) ? 'required' : '' ;
                                $readonly   = (isset($contents_val['readonly']) && $contents_val['readonly'] =='yes' ) ? 'readonly' : '' ;
                                $maxlength  = (isset($contents_val['maxlength']) && $contents_val['maxlength'] != '' ) ? 'maxlength="'.$contents_val['maxlength'].'"' : '' ;
                                
                                if ('valueForCustoms' == $contents_id) {
                                   $default = $product_price_vat;
                                }else if('goodsQuantity' == $contents_id){
                                    $default = $quantity;
                                }else if('valueForCustomsCurrency' == $contents_id){
                                    $default = $currency;
                                }else if('hsTariffNumber' == $contents_id){
                                    $default = $item_tnumber;
                                }else if('descriptionOfContents' == $contents_id){
                                    $default = $item_desc_content;
                                }
                                
                                if (isset($contents_val['display']) && $contents_val['display'] == 'yes' ) {
                                    ?>
                                    <td>
                                        <input 
                                            type="text" 
                                            id="<?php echo esc_attr($contents_id); ?>" 
                                            name="contents[<?php echo esc_attr($item_id); ?>][<?php echo esc_attr($contents_id); ?>]" 
                                            <?php echo esc_attr($req); ?> 
                                            <?php echo esc_attr($readonly); ?> 
                                            <?php echo esc_attr($maxlength); ?> 
                                            value="<?php echo esc_attr($default); ?>"
                                        />
                                    </td>
                                    <?php
                                }
                            }
                            ?>
                            <td><input type="button" value="x" class="posturinn_tariff_deleteRow"></td>
                        </tr>
                    </table>
                </div>
                <?php
                }
                ?>
                </div>
                <?php
            }
            ?>
            
        </div>
    
        <div class="postis-create-shipment-btn-wrap">
            <span class="postis-shipment-create-loader"></span>
            <input type="submit" class="button button-primary" value="<?php _e('Create Shipment', "postis"); ?>"/>
        </div>    
    
    </form>
</div>